with System;
with Ada.Unchecked_Conversion;
with Port_IO;
package body ADC is

   use Common_Units;  -- For operators on Volts

   -- Port addresses of the ADC's two registers
   CSR_Address  : constant Port_IO.Address_Range := 16#262#;
   Data_Address : constant Port_IO.Address_Range := 16#260#;

   Max_Conversion_Time : constant Duration := 10 * 1.0E-3;  -- Ten milliseconds

   type CSR_Rec is  -- Type for the control and status register
      record
         Start      : Boolean;
         Done       : Boolean;
         Int_Enable : Boolean;
         Overflow   : Boolean;
         Gain       : Gain_Choice;
         Channel    : Channel_Number;
      end record;

   for CSR_Rec use
      record
         Start      at 0 range 0 .. 0;
         Done       at 0 range 1 .. 1;
         Int_Enable at 0 range 2 .. 2;
         Overflow   at 0 range 3 .. 3;
         Gain       at 0 range 8 .. 10;
         Channel    at 0 range 11 .. 15;
      end record;

   for CSR_Rec'Size use 16;
   for CSR_Rec'Bit_Order use System.Low_Order_First;

function To_CSR  is new Ada.Unchecked_Conversion
                            (Source => Port_IO.Word,
                             Target => CSR_Rec);
function To_Word is new Ada.Unchecked_Conversion
                            (Source => CSR_Rec,
                             Target => Port_IO.Word);

--------------------------------------------------------------
procedure Read (Channel : in  Channel_Number;
                Gain    : in  Gain_Choice;
                Value   : out Input_Volts) is

-- Port mapped I/O implementation

   Shadow   : CSR_Rec;      -- Copy of control status register
   for Shadow'Size use 16;
   Data_Reg : Port_IO.Word; -- Copy of data register
   for Data_Reg'Size use 16;
begin
   -- Set up for the conversion (using a record aggregate)
   Shadow := (Start      => True,  Done     => False,
              Int_Enable => False, Overflow => False,
              Gain       => Gain,  Channel  => Channel);
   -- Start the conversion (by setting the start bit)
   -- Note conversion of record to word
   Port_IO.Out_Word (Address => CSR_Address,
                     Data    => To_Word (Shadow));
   -- Wait until the conversion is complete
   loop
      delay Max_Conversion_Time / 10;
      -- Get the CSR word from the port
      -- and convert it to a record
      Shadow := To_CSR (Port_IO.In_Word (CSR_Address));
      exit when Shadow.Done;
   end loop;
   if Shadow.Overflow then
      raise Over_Range;
   end if;
   -- Get and convert whole number data to a
   -- fixed point voltage
   Data_Reg := Port_IO.In_Word (Data_Address);
   Value    := 5 * Volts (Data_Reg) / 4095;
end Read;

end ADC;
