with Ada.Containers.Generic_Array_Sort;
with Ada.Numerics.Float_Random;     use Ada.Numerics.Float_Random;
with Ada.Float_Text_IO;             use Ada.Float_Text_IO;
with Ada.Text_IO;                   use Ada.Text_IO;
procedure Concurrent_Sort_Demo is

   type Array_Type is array (Positive range <>) of Float;

   ----------------------------------------------------------------------------
   function Merge (Left  : in Array_Type;
                   Right : in Array_Type) return Array_Type is
   -- Purpose        : Merge two sorted arrays into a single sorted array.
   -- Preconditions  : Left is sorted. Right is sorted.
   -- Postconditions : A sorted array containing all the elements in Left
   --                  and Right is returned.
      Left_Index   : Positive;
      Right_Index  : Positive;
      Result_Index : Positive;
      Result       : Array_Type (1 .. Left'Length + Right'Length);
   begin
      -- Initialize array indexes
      Left_Index   := Left'First;
      Right_Index  := Right'First;
      Result_Index := Result'First;
      loop   -- Each iteration one element is moved into Result
         -- Exit when one array is "empty"
         exit when Left_Index  > Left'Last or
                   Right_Index > Right'Last;
         if Left (Left_Index) < Right (Right_Index) then
            Result (Result_Index) := Left (Left_Index);   -- Copy from Left
            Left_Index := Left_Index + 1;
         else
            Result (Result_Index) := Right (Right_Index); -- Copy from Right
            Right_Index := Right_Index + 1;
         end if;
         Result_Index := Result_Index + 1;
      end loop;

      -- Copy any remaining elements
      if Left_Index <= Left'Last then   -- Still elements remaining in Left?
         Result (Result_Index .. Result'Last) := Left (Left_Index .. Left'Last);
      else
         Result (Result_Index .. Result'Last) := Right (Right_Index .. Right'Last);
      end if;
      return Result;
   end Merge;

   ----------------------------------------------------------------------------
   -- Use a sort from the Ada containers library
   procedure Sort is new Ada.Containers.Generic_Array_Sort
                         (Index_Type   => Positive,
                          Element_Type => Float,
                          Array_Type   => Array_Type,
                          "<"          => Standard."<");

   -----------------------------------------------------------------------------
procedure Concurrent_Sort (Values : in out Array_Type) is
   Split : constant Integer := Values'First + Values'Length / 2;
begin
   declare  -- a block statement
      task type Worker_Task (First : Positive;
                             Last  : Positive);
      task body Worker_Task is
      begin
         -- Call a sequential sort to sort the assigned slice
         Sort (Values (First .. Last));
      end Worker_Task;

      -- Declare two worker tasks
      Low_Part  : Worker_Task (First => Values'First,
                               Last  => Split - 1);
      High_Part : Worker_Task (First => Split,
                               Last  => Values'Last);
   begin  -- executable part of block statement
        -- do nothing (wait for the worker tasks to terminate)
        null;
   end;   -- of block statement

   -- Call a function to merge the two sorted halves;
   Values := Merge (Values (Values'First .. Split - 1),
                    Values (Split .. Values'Last));
end Concurrent_Sort;

   ----------------------------------------------------------------------------
   Generator : Ada.Numerics.Float_Random.Generator;

   subtype Small_Array is Array_Type (1 .. 50);
   Values : Small_Array;

begin
   -- Fill the array with random values
   for Index in Values'Range loop
      Values (Index) := Ada.Numerics.Float_Random.Random (Generator);
   end loop;
   Concurrent_Sort (Values);
   -- Display the sorted array
   for Index in Values'Range loop
      Put (Item => Values (Index),
           Fore => 1,
           Aft  => 4,
           Exp  => 0);
      New_Line;
   end loop;
end Concurrent_Sort_Demo;