with System.Storage_Elements;  use System.Storage_Elements;
with Ada.Interrupts.Names;
package body ADC_Intrpt is

   use Common_Units;  -- For operators on Volts

   -- Addresses of the ADC's two registers (memory mapped I/O)
   CSR_Address  : constant System.Address :=
                              To_Address (16#60008010#);
   Data_Address : constant System.Address :=
                              To_Address (16#60008012#);

   type CSR_Rec is  -- For the control and status register
      record
         Start      : Boolean;
         Done       : Boolean;
         Int_Enable : Boolean;
         Overflow   : Boolean;
         Gain       : Gain_Choice;
         Channel    : Channel_Number;
      end record;
   for CSR_Rec use
      record
         Start      at 0 range 0 .. 0;
         Done       at 0 range 1 .. 1;
         Int_Enable at 0 range 2 .. 2;
         Overflow   at 0 range 3 .. 3;
         Gain       at 0 range 8 .. 10;
         Channel    at 0 range 11 .. 15;
      end record;

   for CSR_Rec'Size use 16;
   for CSR_Rec'Bit_Order use System.Low_Order_First;

   type Data_Type is range 0 .. 4095; -- for the data register
   for Data_Type'Size use 16;

   -- The two device registers
   Control_Status_Register : CSR_Rec;
   for Control_Status_Register'Address use CSR_Address;
   for Control_Status_Register'Size use 16;
   pragma Volatile (Control_Status_Register);

   Data_Register : Data_Type;
   for Data_Register'Address use Data_Address;
   for Data_Register'Size use 16;
   pragma Volatile (Data_Register);

   -----------------------------------------------------------
   -- The interrupt handler is in this protected object
   -----------------------------------------------------------
   protected ADC_Interrupt is
      entry Read (Channel : in  Channel_Number;
                  Gain    : in  Gain_Choice;
                  Value   : out Input_Volts);
   private
      Busy                : Boolean := False;
      Conversion_Complete : Boolean := False;
      procedure Handler;   -- The actual interrupt handler
      pragma Attach_Handler (Handler,
                             Ada.Interrupts.Names.SIGINT);
      pragma Interrupt_Priority;
      entry Wait_For_Completion (Channel : in  Channel_Number;
                                 Gain    : in  Gain_Choice;
                                 Value   : out Input_Volts);
   end ADC_Interrupt;

   protected body ADC_Interrupt is
      entry Read (Channel : in  Channel_Number;
                  Gain    : in  Gain_Choice;
                  Value   : out Input_Volts) when not Busy is
         Shadow : CSR_Rec;
      begin
         -- Set up for the conversion
         Shadow := (Start     => True, Done     => False,
                   Int_Enable => True, Overflow => False,
                   Gain       => Gain, Channel  => Channel);
         -- Start the conversion (by setting the start bit)
         Control_Status_Register := Shadow;
         -- Don't let anyone else execute this entry
         Busy := True;
         requeue Wait_For_Completion; -- Wait for interrupt
      end Read;
      -------------------------------------------
      procedure Handler is
      begin
         Conversion_Complete := True;
      end Handler;
      -------------------------------------------
      entry Wait_For_Completion (Channel : in  Channel_Number;
                                 Gain    : in  Gain_Choice;
                                 Value   : out Input_Volts)
                                   when Conversion_Complete is
      begin
         -- Set up for the next conversion
         Busy := False;
         Conversion_Complete := False;
         -- Check for possible overflow
         if Control_Status_Register.Overflow then
            raise Over_Range;
         end if;
         -- Convert whole number data to a fixed point voltage
         Value := 5 * Volts (Data_Register) / 4095;
      end Wait_For_Completion;
   end ADC_Interrupt;

   --------------------------------------------------------
   procedure Read (Channel : in  Channel_Number;
                   Gain    : in  Gain_Choice;
                   Value   : out Input_Volts) is
   begin
      ADC_Interrupt.Read (Channel, Gain, Value);
   end Read;

end ADC_Intrpt;
