with Broadcasts;
with Ada.Numerics.Float_Random;
with Protected_Output;
procedure Broadcast_Demo is

   type Degrees is delta 0.1 range -100.0 .. 100.0;
   package Broadcast_Temperature is new Broadcasts (Degrees);

   -- The object for broadcasting temperatures
   Degree_Broadcast : Broadcast_Temperature.Broadcast;

   ------------------------------------------------------------------
   protected Random_Duration is
      procedure Get (Item : out Duration);
      -- Returns a random duration value between 0.0 and 2.0 seconds
   private
      First_Call   : Boolean := True;
      My_Generator : Ada.Numerics.Float_Random.Generator;
   end Random_Duration;

   protected body Random_Duration is
      procedure Get (Item : out Duration) is
      begin
         if First_Call then
            -- On the first call, reset the generator
            Ada.Numerics.Float_Random.Reset (My_Generator);
            First_Call := False;
         end if;
         -- Get a random float value and convert it to a duration
         Item := 2 * Duration (Ada.Numerics.Float_Random.Random (My_Generator));
      end Get;
   end Random_Duration;

   ------------------------------------------------------------------
   task type Receiver (ID : Positive);
   task body Receiver is
      Temperature : Degrees;
      My_Delay    : Duration;
   begin
      loop
         -- Wait for a temperature broadcast
         Degree_Broadcast.Tune_In (Temperature);
         -- Display the task ID and received temperature
         Protected_Output.Put_Line ("Task" & Positive'Image (ID) & " received "
                                    & Degrees'Image (Temperature));
         exit when Temperature < 0.0;
         Random_Duration.Get (My_Delay);
         delay My_Delay;
      end loop;
   end Receiver;

   -- A number of receiver tasks
   R1 : Receiver (1);
   R2 : Receiver (2);
   R3 : Receiver (3);
   R4 : Receiver (4);


   Temperature : Degrees := 15.5;
begin
   for Count in 1 .. 20 loop
      delay 0.5;
      Protected_Output.Put_Line ("Broadcasting a temperature");
      Degree_Broadcast.Send (Temperature);
      Temperature := Temperature + 0.5;
   end loop;

   delay 2.5;  -- enough of a delay for all receivers to be waiting
   Protected_Output.Put_Line ("Broadcasting shutdown temperature");
   Degree_Broadcast.Send (-40.0);
end Broadcast_Demo;