with Ada.Integer_Text_IO;        use Ada.Integer_Text_IO;
with Ada.Text_IO;                use Ada.Text_IO;
with Ada.Numerics.Float_Random;  use Ada.Numerics;
with Bounded_Queue;
procedure Producer_Consumer_Demo is

   Number_Of_Producers : constant := 5;
   Producer_Iterations : constant := 8;

   ------------------------------------------------------
   package Integer_Queue is new
           Bounded_Queue (Element_Type => Positive);

   ------------------------------------------------------
   protected type Bounded_Buffer (Max_Size : Positive) is
      procedure Clear;
      -- delete all of the items in the buffer
      entry Put (Item : in Positive);
      -- add a value to the buffer
      entry Take (Item : out Positive);
      -- remove a value from the buffer
   private
      Buffer : Integer_Queue.Queue_Type (Max_Size);
   end Bounded_Buffer;

   protected body Bounded_Buffer is
      procedure Clear is
      begin
         Integer_Queue.Clear (Buffer);
      end Clear;

      entry Put (Item : in Positive)
            when not Integer_Queue.Full (Buffer) is
      begin
         Integer_Queue.Enqueue (Queue => Buffer,
                                Item  => Item);
      end Put;

      entry Take (Item : out Positive)
            when not Integer_Queue.Empty (Buffer) is
      begin
         Integer_Queue.Dequeue (Queue => Buffer,
                                Item  => Item);
      end Take;
   end Bounded_Buffer;

   -- The buffer object.  Holds a maximum of 3 entries.
   The_Buffer : Bounded_Buffer (Max_Size => 3);

   ----------------------------------------------------
   protected ID_Generator is
      procedure Get (ID : out Positive);
      -- Returns a unique positive ID number
   private
      Next_ID : Positive := 1;
   end ID_Generator;

   protected body ID_Generator is
      procedure Get (ID : out Positive) is
      begin
         ID := Next_ID;
         Next_ID := Next_ID + 1;
      end Get;
   end ID_Generator;

   -----------------------------------------------------------
   protected Random_Duration is
      procedure Get (Item : out Duration);
      -- Returns a random duration value
      -- between 0.0 and 1.0 seconds
   private
      First_Call   : Boolean := True;
      My_Generator : Ada.Numerics.Float_Random.Generator;
   end Random_Duration;

   protected body Random_Duration is
      procedure Get (Item : out Duration) is
      begin
         if First_Call then
            -- On the first call, reset the generator
            Ada.Numerics.Float_Random.Reset (My_Generator);
            First_Call := False;
         end if;
         -- Get a random value and convert it to a duration
         Item := Duration (Float_Random.Random (My_Generator));
      end Get;
   end Random_Duration;

   -----------------------------------------------------------
   task type Producer;

   task body Producer is
      My_ID    : Positive;
      My_Delay : Duration;
   begin
      -- Get a unique ID for this task
      ID_Generator.Get (ID => My_ID);
      -- Put my ID into the bounded buffer 8 times
      for Count in 1 .. Producer_Iterations loop
         -- Put my ID into the buffer
         The_Buffer.Put (My_ID);
         -- Simulate the time to do the work
         -- to actually produce something
         Random_Duration.Get (My_Delay);
         delay My_Delay;
      end loop;
   end Producer;

   -- A number of producer tasks
   type Producer_Array is array (1 .. Number_Of_Producers)
                          of Producer;
   Producers : Producer_Array;

   -----------------------------------------------------------
   Value          : Positive;
   Consumer_Delay : Duration;
begin
   -- Consume everything in the bounded buffer
   for Count in 1 .. Number_Of_Producers * Producer_Iterations
   loop
      The_Buffer.Take (Value);
      Put (Item => Value, Width => 2);
      New_Line;
      -- Simulate the time to do the work
      -- to actually consume something
      Random_Duration.Get (Consumer_Delay);
      delay Consumer_Delay / Number_Of_Producers;
   end loop;
end Producer_Consumer_Demo;