with Ada.Text_IO;               use Ada.Text_IO;
with Ada.Numerics.Float_Random; use Ada.Numerics;
procedure Rendezvous_Demo is

   Number_Of_Clients    : constant := 5;
   Number_Of_Iterations : constant := 4;

   -----------------------------------------------------------
   protected Random_Duration is
      procedure Get (Item : out Duration);
      -- Returns a random duration value between
      -- 0.0 and 1.0 seconds
   private
      First_Call   : Boolean := True;
      My_Generator : Ada.Numerics.Float_Random.Generator;
   end Random_Duration;

   protected body Random_Duration is
      procedure Get (Item : out Duration) is
      begin
         if First_Call then
            -- On the first call, reset the generator
            Ada.Numerics.Float_Random.Reset (My_Generator);
            First_Call := False;
         end if;
         -- Get a random float value and
         -- convert it to a duration
         Item := Duration (Float_Random.Random (My_Generator));
      end Get;
   end Random_Duration;

   -----------------------------------------------------------
   task Counter is
      entry Increment (Client : in  Positive;
                       Value  : out Natural);
   end Counter;

   task body Counter is
      Count    : Natural := 0;
      My_Delay : Duration;
   begin
      loop  -- Each iteration, rendezvous with two clients
         Random_Duration.Get (My_Delay);
         delay My_Delay;
         accept Increment (Client : in Positive;
                           Value  : out Natural) do
            Put_Line ("Counter is in #1 rendezvous with task"
                       & Positive'Image (Client));
            Count := Count + 1;
            Value := Count;
         end Increment;
         Random_Duration.Get (My_Delay);
         delay My_Delay;
         accept Increment (Client : in Positive;
                           Value  : out Natural) do
            Put_Line ("Counter is in #2 rendezvous with task"
                      & Positive'Image (Client));
            Count := Count + 1;
            Value := Count;
         end Increment;
      end loop;
   end Counter;

   ----------------------------------------------------------
   task type Client is
      entry Assign_ID (Assigned_ID : in Positive);
   end  Client;

   task body Client is
      My_ID    : Positive;
      My_Delay : Duration;
      Current  : Natural;
   begin
      accept Assign_ID (Assigned_ID : in Positive) do
         My_ID := Assigned_ID;
      end Assign_ID;
      for Count in 1 .. Number_Of_Iterations loop
         Random_Duration.Get (My_Delay);
         delay My_Delay;
         Counter.Increment (Client => My_ID,
                            Value  => Current);
         Put_Line ("Client" & Positive'Image (My_ID) &
                   " got a value of" &
                   Natural'Image (Current));
      end loop;
   end Client;

   type Client_Array is array (1 .. Number_Of_Clients)
                        of Client;
   Clients : Client_Array;

begin
   -- Rendezvous will all the tasks in array Clients
   for Index in Client_Array'Range loop
      -- Rendezvous with a client,
      -- give it the same ID as its index
      Clients (Index).Assign_ID (Assigned_ID => Index);
   end loop;
end Rendezvous_Demo;